/**
    @file ft51_spim.h

    @brief SPI Master Driver

    Driver for the SPI Master hardware.

    @version 1.0.0

**/
/*
 * ============================================================================
 * History
 * =======
 *
 * (C) Copyright 2014, Future Technology Devices International Ltd.
 * ============================================================================
 *
 * This source code ("the Software") is provided by Future Technology Devices
 * International Limited ("FTDI") subject to the licence terms set out
 * http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of FTDI liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on FTDI's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, FTDI
 * has no liability in relation to those amendments.
 * ============================================================================
 */

#ifndef __FT51_SPI_MASTER_H__
#define __FT51_SPI_MASTER_H__

#include <stdint.h>
#include "ft51.h"
#include "ft51_io_registers.h"
#include "registers/ft51_spi_registers.h"

/**
    @brief     Reset SPI Master peripheral
**/
#define SPIM_RESET \
    do { \
        IO_REG_GENERAL_WRITE(SPI_MASTER_CONTROL, MASK_SPI_MASTER_SOFT_RESET); \
    } while (0)

/**
    @brief     Enable SPI Master peripheral
**/
#define SPIM_ENABLE \
    do { \
        IO_REG_GENERAL_WRITE(SPI_MASTER_CONTROL, MASK_SPI_MASTER_DEV_EN); \
    } while (0)

/**
    @brief     Set SPI Master peripheral clock divider
**/
#define SPIM_CLK(sys_clk_divisor) \
    do { \
        IO_REG_GENERAL_WRITE(SPI_MASTER_CLK_DIV, sys_clk_divisor); \
    } while (0)

/**
    @brief     Set SPI Master peripheral polarity, phase and bit order

    Phase and polarity are 2 bits to select mode 0,1,2 or 3.
    Bit order selects MSB first or LSB first.
**/
#define SPIM_SETTINGS(mode, order) \
    do { \
        IO_REG_GENERAL_WRITE(SPI_MASTER_SETUP,  (mode & 0x03) \
                          | (order << 0x02)); \
    } while (0)

/**
    @brief     Set SPI Master peripheral delay

    Sets the delay from SS being activated to starting sending
    data. This is measured in clock cycles.
**/
#define SPIM_DELAY(delay) \
    do { \
        IO_REG_GENERAL_WRITE(SPI_MASTER_DATA_DELAY, delay); \
    } while (0)

/**
    @brief     Set SPI Master peripheral interrupt enables

    Enable either tx_done or rx_full, never both, because they fire at the same time.
**/
#define SPIM_INTERRUPTS(tx_hold, tx_done, tx_overrun, rx_full, rx_overrun, tx_size_done) \
    do { \
        IO_REG_GENERAL_WRITE(SPI_MASTER_IEN, \
                            tx_hold         << 0x00 \
                          | tx_done         << 0x01 \
                          | tx_overrun      << 0x02 \
                          | rx_full         << 0x03 \
                          | rx_overrun      << 0x04 \
                          | tx_size_done    << 0x05); \
    } while (0)

/**
    @brief     Set SPI Master peripheral slave select type

    Select which slave select line to use and whether it is active high or low.
**/
#define SPIM_SS_SETUP(SS_idle_state, SS_num) \
    do { \
        IO_REG_GENERAL_WRITE(SPI_MASTER_SS_SETUP, \
                              SS_idle_state << 0x00 \
                            | SS_num << 0x01); \
    } while (0)

/**
    @brief     Automatically turn SS on in the SPI_MASTER_SETUP register.

    H/L is set depending on the sense of the SS line which is defined in the
    SPI_MASTER_SS_SETUP register.
**/
#define SPIM_SS_ON \
    do { \
        uint8_t value, mask; \
        IO_REG_GENERAL_READ(SPI_MASTER_SETUP, mask); \
        IO_REG_GENERAL_READ(SPI_MASTER_SS_SETUP, value); \
        if (!(value & MASK_SPI_SS_IDLE_STATE)) \
        { \
            mask |= MASK_SPI_SS_N; \
        } \
        else \
        { \
            mask &= (~MASK_SPI_SS_N); \
        } \
        IO_REG_GENERAL_WRITE(SPI_MASTER_SETUP, mask); \
    } while (0)

/**
    @brief     Automatically turn SS off in the SPI_MASTER_SETUP register.

    H/L is set depending on the sense of the SS line which is defined in the
    SPI_MASTER_SS_SETUP register.
**/
#define SPIM_SS_OFF \
    do { \
        uint8_t value, mask; \
        IO_REG_GENERAL_READ(SPI_MASTER_SETUP, mask); \
        IO_REG_GENERAL_READ(SPI_MASTER_SS_SETUP, value); \
        if (value & MASK_SPI_SS_IDLE_STATE) \
        { \
            mask |= MASK_SPI_SS_N; \
        } \
        else \
        { \
            mask &= (~MASK_SPI_SS_N); \
        } \
        IO_REG_GENERAL_WRITE(SPI_MASTER_SETUP, mask); \
    } while (0)

/**
    @enum SPIM_MODE
    @SPI Master mode.
**/
typedef enum
{
    /** Mode 0, CPOL=0, CPHA=0 **/
    SPI_MODE_0,
    /** Mode 1, CPOL=0, CPHA=1 **/
    SPI_MODE_1,
    /** Mode 2, CPOL=1, CPHA=0 **/
    SPI_MODE_2,
    /** Mode 3, CPOL=1, CPHA=1 **/
    SPI_MODE_3,
} SPIM_MODE;

/**
    @enum SPIM_MODE
    @SPI Master bit order.
**/
typedef enum
{
    /** MSB **/
    SPI_BIT_ORDER_MSB,
    /** LSB **/
    SPI_BIT_ORDER_LSB,
} SPIM_BIT_ORDER;


/**
    @brief     \par Initialise the SPI Master to use interrupts.

**/
void SPIM_initialise_ints(void);


/**
    @brief      \par SPI Master Transceive using interrupts.


    Performs a write and a read.
    @note       This function will block indefinitely if called without a
                prior call to SPIM_initialise_ints().
    @param[in]  src    Data to send to the currently-selected Slave.
    @param[out] dest   Receives data from the currently-selected Slave.
                       Must have capacity for (at least) length bytes.
    @param[in]  length Number of bytes to write and read.
**/    
void SPIM_transceive_ints(uint8_t  *src,
                          uint8_t  *dest,
                          uint16_t  length);


/**
    @brief     \par Initialise the SPI Master to use DMA.
**/
void SPIM_initialise_DMA(void);


/**
    @brief      \par SPI Master Transceive using DMA.


    Performs a write and a read.
    @note       This function will block indefinitely if called without a
                prior call to SPIM_initialise_DMA().
    @param[in]  src    Data to send to the currently-selected Slave.
    @param[out] dest   Receives data from the currently-selected Slave.
                       Must have capacity for (at least) length bytes.
    @param[in]  length Number of bytes to write and read.
**/
void SPIM_transceive_DMA(uint8_t             *src,
                         uint8_t             *dest,
                         uint16_t             length);

                         
/**
 * Configure SPI Master to communicate with a specified slave.
 *
 * @param output_signal  IOMUX signal SS_0 to SS_3.
 * @param idle_state 	 Determines if slave-select line should be high
 *                       or low when no slave is selected.
 * @param bit_order      MSB first or LSB first.
 * @param clock_divisor  Specifies the ratio between system clock and SCLK 
 *                       frequencies.  By default, SCLK is half the frequency
 *                       of the system clock; clock_divisor specifies 
 *                       additional divisions.  For example, if the system 
 *                       clock is 12 MHz, a clock_divisor of 75 gives an SCLK
 *                       of 80 kHz.  (12 000000 / (2 * 75) = 80 000.)
 * @param clock_mode     Describes the clock polarity and phase.
 * @param clock_delay    Number of SCLK periods to wait between asserting SS 
 *                       and starting SCLK.  For example, if SCLK is 80 kHz
 *                       a clock_delay of 64 gives a delay of 64 / 80000 =
 *                       800 microseconds.
 */
void SPIM_configure_slave(IOMUX_OUT_SIGNAL  output_signal, 
						  uint8_t 			idle_state,
						  SPIM_BIT_ORDER    bit_order,
                          uint8_t           clock_divisor, 
                          SPIM_MODE         clock_mode, 
                          uint8_t           clock_delay);

#endif // __FT51_SPI_MASTER_H__