/**
    @file tmc_app_usb.h

    @brief Header file for USB functions for TMC Class demo on FT51.
 *

    @version 1.0.0

**/
/*
 * ============================================================================
 * History
 * =======
 *
 * (C) Copyright 2014, Future Technology Devices International Ltd.
 * ============================================================================
 *
 * This source code ("the Software") is provided by Future Technology Devices
 * International Limited ("FTDI") subject to the licence terms set out
 * http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of FTDI liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on FTDI's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, FTDI
 * has no liability in relation to those amendments.
 * ============================================================================
 */

#ifndef __TMC_APP_USB_H__
#define __TMC_APP_USB_H__

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/* INCLUDES ************************************************************************/
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <stdio.h>

#include "ft51_usb.h"
#include "ft51_usb_dfu.h"
#include "ft51_usb_hid.h"
#include "ft51_usb_tmc.h"
#include "ft51_dfu.h"

#include "ft51_uart.h"
#include "ft51_general_config.h"

/* CONSTANTS ***********************************************************************/
/**
 @name USB and Hub Configuration
 @brief Indication of whether the USB Hub is enabled, how it is
        powered and the size of the control endpoint Max Packets.
 */
//@{
// USB Hub Enabled - set to 1 for enabled and 0 for disabled
#ifndef USB_HUB_ENABLED
#define USB_HUB_ENABLED 1
#endif // USB_HUB_ENABLED
// USB Self or Bus Powered - set to 1 for self-powered or 0 for bus-powered
#ifndef USB_SELF_POWERED
#define USB_SELF_POWERED 1
#endif // USB_SELF_POWERED
// USB Remote Wakeup - set to 1 for enabled and 0 for disabled
#ifndef USB_REMOTE_WAKEUP
#define USB_REMOTE_WAKEUP 0
#endif // USB_REMOTE_WAKEUP

#if USB_SELF_POWERED == 1
#if USB_REMOTE_WAKEUP == 1
#define USB_CONFIG_BMATTRIBUTES_VALUE (USB_CONFIG_BMATTRIBUTES_SELF_POWERED | USB_CONFIG_BMATTRIBUTES_REMOTE_WAKEUP | USB_CONFIG_BMATTRIBUTES_RESERVED_SET_TO_1)
#else // USB_REMOTE_WAKEUP
#define USB_CONFIG_BMATTRIBUTES_VALUE (USB_CONFIG_BMATTRIBUTES_SELF_POWERED | USB_CONFIG_BMATTRIBUTES_RESERVED_SET_TO_1)
#endif // USB_REMOTE_WAKEUP
#else // USB_SELF_POWERED
#if USB_REMOTE_WAKEUP == 1
#define USB_CONFIG_BMATTRIBUTES_VALUE (USB_CONFIG_BMATTRIBUTES_REMOTE_WAKEUP | USB_CONFIG_BMATTRIBUTES_RESERVED_SET_TO_1)
#else // USB_REMOTE_WAKEUP
#define USB_CONFIG_BMATTRIBUTES_VALUE USB_CONFIG_BMATTRIBUTES_RESERVED_SET_TO_1
#endif // USB_REMOTE_WAKEUP
#endif // USB_SELF_POWERED

// USB Endpoint Zero packet size (both must match)
#define USB_CONTROL_EP_MAX_PACKET_SIZE 16
#define USB_CONTROL_EP_SIZE USB_EP_SIZE_16

// USB Bulk Endpoint packet size (both must match)
#define USB_BULK_EP_MAX_PACKET_SIZE 64
#define USB_BULK_EP_SIZE USB_EP_SIZE_64
//@}

/**
 @name Device Configuration Areas
 @brief Size and location of string descriptors and
 structure used to modify device and configuration
 descriptors.
 */
//@{
// String descriptors - allow a maximum of 256 bytes for this
#define STRING_DESCRIPTOR_LOCATION 0x80
#define STRING_DESCRIPTOR_ALLOCATION 0x100
//@}

/**
 @name USB_PID_DEMO configuration.
 @brief FTDI predefined TMC Demo Product ID.
 */
//@{
#define USB_PID_DEMO 0x0feb
//@}

/**
 @name Firmware Functionality
 @brief Enable or disable functions of the firmware.
 */
//@{
/// Enable (non-zero) or disable (zero) TMC support.
#ifndef TMC
#define TMC 1
#endif // TMC
/// Enable (non-zero) or disable (zero) DFU firmware updates.
#ifndef DFU
#define DFU 1
#endif // DFU
/// Enable (non-zero) or disable (zero) device detach-attach
/// sequences in DFU firmware.
/// This is used in the bmAttributes of the
/// USB_dfu_functional_descriptor.
#ifndef DFU_DETACH_MODE
#define DFU_DETACH_MODE 1
#endif // DFU_DETACH_MODE
/// Enable (non-zero) or disable (zero) LCD display.
#ifndef LCD
#define LCD 0
#endif // LCD
/// Enable (non-zero) or disable (zero) Force Measurement
#ifndef MEAS_FORCE
#define MEAS_FORCE 1
#endif // MEAS_FORCE
/// Enable (non-zero) or disable (zero) Heart Rate Measurement.
#ifndef MEAS_HR
#define MEAS_HR 1
#endif // MEAS_HR
/// Enable (non-zero) or disable (zero) Temperature Measurement.
#ifndef MEAS_TEMP
#define MEAS_TEMP 1
#endif // MEAS_TEMP
//@}

/**
 @name USB_PID_DEMO configuration.
 @brief Run Time Product ID for demo function.
 */
//@{

#ifdef DFU
/**
 @name DFU_BLOCK_SIZE definition.
 @brief Number of bytes in block, sent in each DFU_DNLOAD request
 from the DFU update program on the host. This is simplified
 in that the meaning of a block is an arbitrary number of
 bytes. This is intentionally a multiple of the maximum
 packet size for the control endpoints.
 */
//@{
#define DFU_BLOCK_SIZE 64

//@}

/**
 @name DFU_USB_PID_DFUMODE configuration.
 @brief FTDI predefined DFU Mode Product ID.
 */
//@{
#define DFU_USB_PID_DFUMODE 0x0fee
//@}

/**
 @name DFU_USB_INTERFACE configuration..
 @brief Run Time and DFU Mode Interface Numbers.
 */
//@{
#define DFU_USB_INTERFACE_RUNTIME 1
#define DFU_USB_INTERFACE_DFUMODE 0
//@}
#endif /* DFU */

/* TYPES ***************************************************************************/

/* GLOBAL VARIABLES ****************************************************************/

/* MACROS **************************************************************************/

/* FUNCTION PROTOTYPES *************************************************************/
void usb_setup(void);


#ifdef __cplusplus
} /* extern "C" */
#endif /* __cplusplus */

#endif /* __TMC_APP_USB_H__ */